!
! matrixfile --- Manage a sparse matrix in a file
! Christian Buth
! 18 Sep 2006
!
!
! Communication file for the properties of the matrix
! Assumed structure of 'Filecom':
!
!         signature               ! Identify communication file
!         main                    ! Dimension of main space of matrix, i.e.
!                                 ! the number of vectors to be used in a block
!                                 ! Lanczos operation
!         nrecd                   ! Number of records written to the file Filedia
!         nreco                   ! Number of records written to the file Fileoff
!         ndim                    ! Dimension of the matrix
!         nbuf                    ! Buffer size in Fileoff
!         numdiag                 ! Number of diagonal matrix elements
!         numoff                  ! Number of off-diagonal matrix elements
!         threshold               ! Threshold for writing matrix elements
!
!
! with the types of the variables being
!
!         character(len=17) signature
!         integer main, nrecd, nreco, ndim, nbuf, numdiag, numoff
!         real threshold
!
!
! File for the diagonal matrix elements of the matrix
! Assumed structure of 'Filedia':
!
!         diag(nbuf)              ! Diagonal matrix elements of the matrix
!                                 ! Extension to Francescos Tarantellis file format
!         idx(nbuf)               ! Chunk of column indices
!         nwd                     ! Number of datums in the record
!
!
! with the types of the variables being
!
!         complex(dblprec) diag
!         integer idx, nwd
!
! for nrecd records; nrecd, nbuf are given in the communication file
! of the matrix.
!
!
! File for the off-diagonal matrix elements of the matrix
! Assumed structure of 'Fileoff':
!
!         off(nbuf)               ! Chunk of matrix elements of the matrix
!         iox(nbuf)               ! Chunk of column indices
!         jox(nbuf)               ! Chunk of row indices
!                                 ! Extension to Francescos Tarantellis file format
!         nwo                     ! Number of datums in the record
!
! with the types of the variables being
!
!         complex(dblprec) off
!         integer iox, jox, nwo
!
! for nreco records; nreco, nbuf are given in the communication file of the matrix.
!


!
! Module
!

      module matrixfile
      use fortrantl
      implicit none


!
! Parameters
!

      character(17), parameter :: matrixfilesignature = 'matrixfile_V1.0.0'


!
! Types
!

      type tmatrixfile
        logical readfile                        ! Read file (.true.) or write file (.false.)

        integer fmtxcom,     &                  ! Unit number for the communication file for the matrix (Filecom)
     &          fmtxdia,     &                  ! Unit number for the file of diagonal matrix elements (Filedia)
     &          fmtxoff,     &                  ! Unit number for the file of off-diagonal matrix elements (Fileoff)
     &          main,        &                  ! Dimension of the main space, the number of Lanczos vectors
     &          nrecd,       &                  ! Number of records in Filedia
     &          nreco,       &                  ! Number of records in Fileoff
     &          ndim,        &                  ! Dimension of the matrix
     &          nbuf,        &                  ! Buffer size of Fileoff
     &          nwd,         &                  ! Number of entries in the current record of Filedia
     &          nwo,         &                  ! Number of entries in the current record of Fileoff
     &          currrecd,    &                  ! Presently processed record of Filedia
     &          currreco,    &                  ! Presently processed record of Fileoff
     &          numdiag,     &                  ! Number of diagonal elements in Filedia while reading,
                                                ! Number of diagonal elements written to Filedia while writing
     &          numoff                          ! Number of off-diagonal elements in Fileoff while reading,
                                                ! Number of off-diagonal elements written to Fileoff while writing

        integer, pointer :: idx(:),  &
     &                      iox(:),  &
     &                      jox(:)

        real threshold                          ! Threshold for writing matrix elements to disk

        complex(dblprec), pointer :: diag(:), off(:)
      end type tmatrixfile


      contains


!
! openmatrixfile --- open an initialize files for diagonal and off-diagonal
!               elements of the matrix
!

      subroutine openmatrixfile(mf)
      implicit none

      type(tmatrixfile), intent(inout) :: mf
      character(len(matrixfilesignature)) signature
      integer rld, rlo

      ! Write files
      if (.not. mf%readfile) then

        ! Initialize file writing
        allocate(mf%diag(mf%nbuf), mf%idx(mf%nbuf),  &
     &           mf%off (mf%nbuf), mf%iox(mf%nbuf), mf%jox(mf%nbuf))

        ! Determine record lengths
        rld = (16 + 4    ) * mf%nbuf + 4
        rlo = (16 + 4 + 4) * mf%nbuf + 4

        mf%nrecd    = 0
        mf%nreco    = 0
        mf%nwd      = 0
        mf%nwo      = 0
        mf%currrecd = 0
        mf%currreco = 0
        mf%numdiag  = 0
        mf%numoff   = 0

        ! Open files for the diagonal and the off-diagonal matrix
        ! elements of the matrix
        open(unit=mf%fmtxcom, file='Filecom', status='replace', &
     &       action='write', form='formatted', access='sequential')
        open(unit=mf%fmtxdia, file='Filedia', status='replace', &
     &       action='write', form='unformatted', access='direct', recl=rld)
        open(unit=mf%fmtxoff, file='Fileoff', status='replace', &
     &       action='write', form='unformatted', access='direct', recl=rlo)


      ! Read files
      else

        ! Read communication file for the matrix
        open(unit=mf%fmtxcom, file='Filecom', status='old', &
     &       action='read', form='formatted', access='sequential')
        read (mf%fmtxcom,*) signature
        if (signature /= matrixfilesignature) then
          write (*,*) 'Filecom is not a valid '//matrixfilesignature//' file!'
          stop 'matrixfile: Error in openmatrixfile'
        end if
        read (mf%fmtxcom,*) mf%main, mf%nrecd, mf%nreco, mf%ndim, &
     &          mf%nbuf, mf%numdiag, &
     &          mf%numoff, mf%threshold
        close (mf%fmtxcom)

        allocate(mf%diag(mf%nbuf), mf%idx(mf%nbuf), &
     &           mf%off(mf%nbuf), mf%iox(mf%nbuf), mf%jox(mf%nbuf))

        mf%nwd = 0
        mf%nwo = 0
        mf%currrecd = 0
        mf%currreco = 0

        ! Determine record lengths
        rld = (16 + 4) * mf%nbuf + 4
        rlo = (16 + 4 + 4) * mf%nbuf + 4

        ! Initialize file writing
        write (*,*) 'main, nrecd, nreco, ndim, recld, reclo, nbuf, ', &
     &              'numdiag, numoff, threshold'
        write (*,*) mf%main, mf%nrecd, mf%nreco, mf%ndim, rld, rlo, mf%nbuf,  &
     &              mf%numdiag, mf%numoff, mf%threshold

        ! Open files for the diagonal and the off-diagonal matrix
        ! elements of the matrix
        open(unit=mf%fmtxdia, file='Filedia', status='old', &
     &       action='read',form='unformatted', access='direct', recl=rld)
        open(unit=mf%fmtxoff, file='Fileoff', status='old', &
     &       action='read', form='unformatted', access='direct', recl=rlo)

      end if
      end subroutine openmatrixfile


!
! closematrixfile --- close files for diagonal and off-diagonal elements
!               of the matrix
!

      subroutine closematrixfile(mf)
      implicit none

      type(tmatrixfile), intent(inout) :: mf

      if (.not. mf%readfile) then
        call writematrixbuffer(mf, .true.)

        write (mf%fmtxcom,*) matrixfilesignature
        write (mf%fmtxcom,*) mf%main, mf%nrecd, mf%nreco, mf%ndim, &
     &                    mf%nbuf, mf%numdiag, &
     &                    mf%numoff, mf%threshold
        close (mf%fmtxcom)

        write (*,*) 'main, nrecd, nreco, ndim, nbuf, ', &
     &              'numdiag, numoff, threshold'
        write (*,*) mf%main, mf%nrecd, mf%nreco, mf%ndim, mf%nbuf,  &
     &              mf%numdiag, mf%numoff, mf%threshold
      end if
      close (mf%fmtxdia)
      close (mf%fmtxoff)

      ! Deallocate memory
      deallocate(mf%diag, mf%off, mf%idx, mf%iox, mf%jox)
      end subroutine closematrixfile


!
! writeentry --- write matrix element to the matrix file
!

      subroutine writeentry(mf, i, j, elem)
      implicit none

      integer, intent(in) :: i, j
      complex(dblprec), intent(in) :: elem
      type(tmatrixfile), intent(inout) :: mf

      if (i == j) then
        call writediagentry(mf, i, elem)
      else
        call writeoffdiagentry(mf, i, j, elem)
      end if
      end subroutine writeentry


!
! writediagentry --- write diagonal matrix element to the matrix file
!

      subroutine writediagentry(mf, i, delem)
      implicit none

      integer, intent(in) :: i
      complex(dblprec), intent(in) :: delem
      type(tmatrixfile), intent(inout) :: mf

      if (abs(delem) < mf%threshold) then
        return
      end if

      mf%nwd = mf%nwd + 1
      mf%numdiag = mf%numdiag + 1
      mf%diag(mf%nwd) = delem
      mf%idx(mf%nwd) = i

      if (mf%nwd == mf%nbuf) then
        call writematrixbuffer(mf, .false.)
      end if
      end subroutine writediagentry


!
! writeoffdiagentry --- write off-diagonal matrix element to the
!       matrix file
!

      subroutine writeoffdiagentry(mf, i, j, oelem)
      implicit none

      integer, intent(in) :: i, j
      complex(dblprec), intent(in) ::  oelem
      type(tmatrixfile), intent(inout) :: mf

      if (abs(oelem) < mf%threshold) then
        return
      end if

      if (i == j) then
        stop 'writeoffdiagentry: diagonal element passed!'
      end if

      mf%nwo = mf%nwo + 1
      mf%numoff = mf%numoff + 1
      mf%off(mf%nwo)  = oelem
      mf%iox(mf%nwo) = i
      mf%jox(mf%nwo) = j

      if (mf%nwo == mf%nbuf) then
        call writematrixbuffer(mf, .false.)
      end if
      end subroutine writeoffdiagentry


!
! writematrixbuffer --- write chunk of off-diagonal matrix elements to the
!               matrix file
!

      subroutine writematrixbuffer(mf, forceflush)
      implicit none

      logical, intent(in) :: forceflush
      type(tmatrixfile), intent(inout) :: mf

      ! Flush buffer of the diagonal matrix elements of the matrix
      if (forceflush .or. mf%nwd == mf%nbuf) then
        mf%nrecd = mf%nrecd + 1
        write (unit=mf%fmtxdia, rec=mf%nrecd) mf%diag, mf%idx, mf%nwd
        mf%nwd = 0
      end if

      ! Flush buffer of the off-diagonal matrix elements of the matrix
      if (forceflush .or. mf%nwo == mf%nbuf) then
        mf%nreco = mf%nreco + 1
        write (unit=mf%fmtxoff, rec=mf%nreco) mf%off, mf%iox, mf%jox, mf%nwo
        mf%nwo = 0
      end if
      end subroutine writematrixbuffer


!
! readmatrixbuffer --- read chunk of off-diagonal matrix elements to the
!               matrix file
!

      subroutine readmatrixbuffer(mf, diagoff, mfeof)
      implicit none

      logical, intent(in) :: diagoff
      logical, intent(out) :: mfeof
      type(tmatrixfile), intent(inout) :: mf

      if (diagoff) then

        mf%currrecd = mf%currrecd + 1
        if (mf%currrecd > mf%nrecd) then
          mfeof = .true.
        else
          read (unit=mf%fmtxdia, rec=mf%currrecd) mf%diag, mf%idx, mf%nwd
          mfeof = .false.
        end if

      else

        mf%currreco = mf%currreco + 1
        if (mf%currreco > mf%nreco) then
          mfeof = .true.
        else
          read (unit=mf%fmtxoff, rec=mf%currreco) mf%off, mf%iox, mf%jox, mf%nwo
          mfeof = .false.
        end if

      end if
      end subroutine readmatrixbuffer


!
! mfrewind --- rewind files for rereading
!

      subroutine mfrewind(mf)
      implicit none

      type(tmatrixfile), intent(inout) :: mf

      mf%currrecd = 0
      mf%currreco = 0
      mf%nwd = 0
      mf%nwo = 0

      ! As fmtxdia and fmtxoff are direct access files, an
      ! explicit rewind is not necessary and not allowed
!       rewind(mf%fmtxdia)
!       rewind(mf%fmtxoff)
      end subroutine mfrewind
      end module matrixfile