!
! oneel --- one-electron, fine-element matrix generator
! Mark Baertschy, Kevin Christ, Robin Santra, Christian Buth
! 5 May 2006
!
!
! This program gives the energy eigenvalues and radial eigenfunctions for a
! one-electron potential
!
! Version V1.0.0: 23 Jun 2006 (Namelist support added, smooth exterior
!       complex scaling CAPs implemented)
! Version V1.1.0: 15 Nov 2006 (Analytical derivatives, new numerical
!       integration routines)
!


      program oneel
      use FiniteElements
      use SESpath
      implicit none

      integer, allocatable :: ndcnt(:), ndcntfin(:,:)
      integer leftright(2), nodestot, nbasis, nstate, nl,   &
     &          ngrid, nvlnc, npotpts, ipottype, it0, it1,  &
     &          ict, ntot, j, k
      real(dblprec), allocatable :: Eng(:,:), Cevec(:,:,:), egfxn(:,:,:), egder(:,:,:)
      real(dblprec) rf, xh, &
     &          theta,      &           ! Complex scaling angle
     &          lambda,     &           ! Smoothness of the path
     &          r0,         &           ! Start distance
     &          lastpotentialvalue,  &
     &              ionr                ! Beginning with this distance from the
                ! center, the one-electron potential of the atom is that of
                ! a single elementary charge
      type(FEBnodes) xnd

      namelist/OneElectron/ rf, nodestot, nvlnc, nl, nstate, theta,  &
     &          lambda, r0, npotpts, ngrid, ipottype, originCS

      ! Initialization
      write (*,*) 'One-electron V1.1.1, finite-element matrix generator'
      write (*,*) '(c) 2006 Mark Baertschy, Kevin Christ, Chris H. Greene, Robin Santra, Christian Buth'
      write (*,*)

      ! Default values for parameters
      rf       = 60.0D0
      nodestot = 1001
      nvlnc    = 1
      nl       = 3
      nstate   = 50
      theta    = 0.13D0
      lambda   = 5.0D0
      r0       = 7.0D0
      npotpts  = 441
      ngrid    = 3 * nodestot
      ipottype = 0                      ! Read in point grid
      originCS = .false.                ! Use SES instead of CS from the origin

      read(*,nml=OneElectron)

      ! Adjust Number of points for numerical quadrature rules
      ngrid = ngrid - mod(ngrid, 3) + 1         ! For 3/8-rule
      if (.not. btest(ngrid, 0)) then           ! Odd number for Simpson's rule
        ngrid = ngrid + 3
      end if

      write(*,*) 'Number of points used to discretize the atomic ',  &
     &           'potential: npotpts =', npotpts

      call initSESpath(theta, lambda, r0)
      call initFiniteElements(npotpts)


!! LEFTRIGHT
!! leftright(2) enforces zero boundary conditions at either end of grid.
!! It is the optional last argument to CreateNodes.
!! If leftright argument is not specified then CreateNodes assumes a value
!!  of 2 for both elements of leftright.
!! This default value forces the function (but not its derivatives) to
!!  be zero at both ends of the grid (closed channels).

      leftright(1) = 2          ! include only f.e. types 1 and 2 (exclude type 0) on left
      leftright(2) = 2          ! include only f.e. types 1 and 2 (exclude type 0) on right


!! SQUARE ROOT MESH
!! Creates a square root mesh for the finite elements
!! The mesh starts at r=0
!! rf is the maximum value of the grid and the location of the last node
!! nodestot is the desired number of nodes
!! xnd is the output data structure containing information about
!!  finite element nodes
!! nbasis is the output number of basis functions

      write(*,*) 'Extension of the grid, rf =', rf
      call sqrtmesh(rf, nodestot, xnd, nbasis, leftright)


!! CALCULATE EIGENVECTORS AND EIGENVALUES FOR ALL l AND j

      call system_clock(it0, ict)

!! eigencalc calls an ARPACK routine to calculate eigenvalues and eigenvectors
!!  and calls aevmtrx, which creates the potential energy matrix

!! nvlnc is the number of the S shell of the valence electron
!! calculation of atomic orbitals starts with this shell

!! nl is the number of l quantum numbers starting with l=0
!! nstate is the total number of (closed) eigenstates desired per l

!! ntot is the total number of eigenstates

!! Eigenvalues are returned in Eng, and the eigenvectors are returned in Cevec
!! ndcnt keeps track of the amount of wavefunction nodes in the ground state
!!  of each l

      write(*,*) 'Number of the S shell of the valence electron, nvlnc =', nvlnc
      ntot = nstate * nl
      write(*,*) 'Eigenbasis with nl =', nl, ', nstate =', nstate, ', ntot =', ntot

      allocate(Eng(nstate, nl), Cevec(nbasis, nstate, nl), ndcnt(nl))
      call eigencalc(nbasis, nstate, nl, ntot, nvlnc, Eng, Cevec, ndcnt,  &
     &          xnd, ionr, lastpotentialvalue, ipottype)

      call system_clock(it1)
      write(*,*) 'ARPACK eigenvalue routine time is', ((it1-it0) * 1.0D0 / ict)


!! FUNCTION EVALUATION

!! Evaluate eigenfunctions on a grid
!! ngrid is the number of grid points fitted onto a grid of
!! size rf (from above)

      allocate(egfxn(ngrid, nstate, nl), egder(ngrid, nstate, nl))
      call eigenfxn(Cevec, ngrid, rf, xh, nbasis, nstate, nl, xnd, egfxn, egder)

      allocate(ndcntfin(nstate, nl))
      do k = 1, nl
        do j = 1, nstate
          ndcntfin(j,k) = ndcnt(k) + (j-1)
        end do
      end do


!! MATRIX CREATION

!! Create matrix representations of the dipole operator
!! in the atomic eigenvector basis
!! Energies and matrix elements are written to "atom.dat"

      call mtrxbuild(Cevec, nbasis, nstate, nvlnc, ntot, nl, xnd, Eng,  &
     &          ndcntfin, ngrid, egfxn, egder, xh, ionr,          &
     &          lastpotentialvalue, ipottype)

      deallocate(Eng, Cevec, ndcnt, egfxn, egder, ndcntfin)
      call exitSESpath
      call exitFiniteElements

      write (*,*)
      write (*,*) 'Vive la radiación!'
      end program oneel
